library(matlab) # calling for "meshgrid"
##############################################################
## File: AbsoluteAR.R
## Coded by: Dong Li
##
## This program is for an absolute AR(1) model:
##    Y_{t} = -alpha |Y_{t-1}| + e_{t}
##
## Reference:
## Li, D. and Qiu, J. (2020).
##   The marginal density of a TMA(1) process. 
##   Journal of Time Series Analysis, 41, 476--484.
##############################################################
##########  The function K(x, y)  ############################
##############################################################
K.fun = function(z,alpha){
f     = pnorm(z$x+alpha*abs(z$y))
return(f)
}
##############################################################
##############################################################
g.den = function(x,XX,G,alpha){
dif   = c(G,1)-c(0,G)
fden  = sum(dnorm(x+alpha*abs(XX))*dif)
return(fden)
}
##############################################################
alpha = 0.5  # AR(1) parameter
x     = seq(-4,4,by=0.01)
n     = length(x)
y     = (c(min(x)-1,x)+c(x, max(x)+1))/2
z     = meshgrid(x,y)

A=t(K.fun(z,alpha))
K = A[,1:n]-A[,2:(n+1)]
a = A[,n+1]
Ab.AR.G     = solve(eye(n)-K)%*%a
Ab.AR.g.den = sapply(x,g.den,XX=c(x,max(x)+1),G=Ab.AR.G,alpha)

############# Plot the density function ##########################
par(mfrow=c(1,2))
plot(x,Ab.AR.g.den,type="l",col="red",las=1,lwd=3,cex.lab=1.3,cex.axis=1.3,
  xlab="x",ylab="",main=expression(paste("(a)  ", alpha==0.5),adj=0),
  cex.main=2,ylim=c(0,0.4))
abline(h=c(0,1),lty=2)

Tden=exp(-(1-alpha^2)*x^2/2)*pnorm(-abs(alpha)*x)*sqrt(2*(1-alpha^2)/pi)
lines(x,Tden,lty=2,lwd=3,col="cyan")
legend(1,0.4,c("Ours","Theoretical"),lty=c(1,2),lwd=3,col=c("red","cyan"),cex=1.5)
##############################################################
##############################################################
alpha = 0.9  # AR(1) parameter
x    = seq(-8,4,by=0.01)
n    = length(x)
y    = (c(min(x)-1,x)+c(x, max(x)+1))/2
z    = meshgrid(x,y)

A = t(K.fun(z,alpha))
K = A[,1:n]-A[,2:(n+1)]
a = A[,n+1]
Ab.AR.G     = solve(eye(n)-K)%*%a
Ab.AR.g.den = sapply(x,g.den,XX=c(x,max(x)+1),G=Ab.AR.G,alpha)

############# Plot the density function ##########################
plot(x,Ab.AR.g.den,type="l",col="red",las=1,lwd=3,cex.lab=1.3,cex.axis=1.3,
  xlab="x",ylab="",main=expression(paste("(b)  ", alpha==0.9),adj=0),lty=1,
  cex.main=2,ylim=c(0,0.4))
abline(h=c(0,1),lty=2)

Tden=exp(-(1-alpha^2)*x^2/2)*pnorm(-abs(alpha)*x)*sqrt(2*(1-alpha^2)/pi)
lines(x,Tden,lty=2,lwd=3,col="cyan")
legend(-8,0.4,c("Ours","Theoretical"),lty=c(1,2),lwd=3,col=c("red","cyan"),cex=1.5)
##############################################################
##############################################################
##############################################################
##############################################################
##############################################################
##############################################################
############# Plot the distribution function ##########################
#plot(x,Ab.AR.G,type="l",col="blue",las=1,lwd=2,cex.lab=1.3,cex.axis=1.3,
#  xlab="x",ylab="",main="The distribution ")
#abline(h=c(0,1),lty=2)
