library(matlab)  # calling for "meshgrid"
##############################################################
## R-code EXPAR.R
##
## This program is for EXPAR model:
## Y_{t} = phi + [alpha + beta *exp(-gamma Y^{2}_{t-1})]Y_{t-1} + e_{t}
##
## Coded by: Dong Li
##
## Reference:
## Li, D. and Qiu, J. (2020).
##   The marginal density of a TMA(1) process. 
##   Journal of Time Series Analysis, 41, 476--484.
##############################################################
##########  Sampling  STAR model  ############################
##############################################################
EXPAR.sim = function(n,th){
m     = 200
error = rnorm(m+n)
y     = numeric(m+n)
for (i in 2:(m+n)){
 y[i]=th[1]+(th[2]+th[3]*exp(-th[4]*y[i-1]^2))*y[i-1]+error[i]
 }
return(y[-(1:m)])
}
##############################################################
##########  The function K(x, y)  ############################
##############################################################
K.fun = function(z,th){
f     = pnorm(z$x-(th[1]+(th[2]+th[3]*exp(-th[4]*(z$y)^2))*z$y))
return(f)
}
##############################################################
##############################################################
g.den = function(x,XX,G,th){
dif   = c(G,1)-c(0,G)
fden  = sum(dnorm(x-(th[1]+(th[2]+th[3]*exp(-th[4]*(XX)^2))*XX))*dif)
return(fden)
}
##############################################################
th = c(0,0.5,1.5,0.1)  # parameters: phi, alpha, beta, gamma
x  = seq(-6,6,by=0.01)
n  = length(x)
y  = (c(min(x)-1,x)+c(x, max(x)+1))/2
z  = meshgrid(x,y)

A = t(K.fun(z,th))
K = A[,1:n]-A[,2:(n+1)]
a = A[,n+1]
EXPAR.G     = solve(eye(n)-K)%*%a
EXPAR.g.den = sapply(x,g.den,XX=c(x,max(x)+1),G=EXPAR.G,th)

############# Plot the density function ##########################
par(mfrow=c(2,2))
plot(x,EXPAR.g.den,type="l",col="blue",las=1,lwd=3,cex.lab=1.3,cex.axis=1.3,
  xlab="x",ylab="",main=expression(paste("(a)  ", gamma==0.1),adj=0),
  cex.main=2,ylim=c(0,0.4))
abline(h=c(0,1),lty=2)


##############################################################
##############################################################
##############################################################
th = c(0,0.5,1.5,0.5)  # parameters: phi, alpha, beta, gamma
x  = seq(-6,6,by=0.01)
n  = length(x)
y  = (c(min(x)-1,x)+c(x, max(x)+1))/2
z  = meshgrid(x,y)

A = t(K.fun(z,th))
K = A[,1:n]-A[,2:(n+1)]
a = A[,n+1]
EXPAR.G = solve(eye(n)-K)%*%a
EXPAR.g.den = sapply(x,g.den,XX=c(x,max(x)+1),G=EXPAR.G,th)

############# Plot the density function ##########################
plot(x,EXPAR.g.den,type="l",col="blue",las=1,lwd=3,cex.lab=1.3,cex.axis=1.3,
  xlab="x",ylab="",main=expression(paste("(b)  ", gamma==0.5),adj=0),
  cex.main=2,ylim=c(0,0.4))
abline(h=c(0,1),lty=2)

##################################################################
th = c(0,0.5,1.5,0.8)  # parameters: phi, alpha, beta, gamma
x  = seq(-6,6,by=0.01)
n  = length(x)
y  = (c(min(x)-1,x)+c(x, max(x)+1))/2
z  = meshgrid(x,y)

A = t(K.fun(z,th))
K = A[,1:n]-A[,2:(n+1)]
a = A[,n+1]
EXPAR.G = solve(eye(n)-K)%*%a
EXPAR.g.den = sapply(x,g.den,XX=c(x,max(x)+1),G=EXPAR.G,th)

############# Plot the density function ##########################
plot(x,EXPAR.g.den,type="l",col="blue",las=1,lwd=3,cex.lab=1.3,cex.axis=1.3,
  xlab="x",ylab="",main=expression(paste("(c)  ", gamma==0.8),adj=0),
  cex.main=2,ylim=c(0,0.4))
abline(h=c(0,1),lty=2)

##################################################################
th = c(0,0.5,1.5,1)   # parameters: phi, alpha, beta, gamma
x  = seq(-6,6,by=0.01)
n  = length(x)
y  = (c(min(x)-1,x)+c(x, max(x)+1))/2
z  = meshgrid(x,y)

A = t(K.fun(z,th))
K = A[,1:n]-A[,2:(n+1)]
a = A[,n+1]
EXPAR.G = solve(eye(n)-K)%*%a
EXPAR.g.den = sapply(x,g.den,XX=c(x,max(x)+1),G=EXPAR.G,th)

############# Plot the density function ##########################
plot(x,EXPAR.g.den,type="l",col="blue",las=1,lwd=3,cex.lab=1.3,cex.axis=1.3,
  xlab="x",ylab="",main=expression(paste("(d)  ", gamma==1),adj=0),
  cex.main=2,ylim=c(0,0.4))
abline(h=c(0,1),lty=2)




