% Matlab file: pdf_Uniform_discretization.m
% 
% Uniform innovation distribution
%
% input file: stationary.m (see pdf_ARCH_Gaussian_discretization.m)
%
% Reference:
%  Herve, L. and Ledoux, J. (2020). State-discretization of 
%    V-geometrically ergodic Markov chains and convergence to
%    stationarity,
%    Methodology and Computing in Applied Probability,
%    22: 905--925.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all; clc;

% Set the value of "rho" (default): 
rho1 = 0.9;

% If fixed at order 3 the Markov chain has the matrix "P^3" and "rho" must be
% replaced by "rho^3"
  rho = (rho1)^3;

% Define the error processes:
% nu = @(x) pdf('unif',x,0,1);    % nu_{1} or h_{0}
% fplot(nu,[-0.5,1.5]) 
% support = [0,1] or the support is [0,2] for rho=0.5, 
%           [0,10/3] for  rho=0.7; [0,10] for rho=0.9 

% nu = @(x) x.*(x >=0 & x<rho1) +(1-rho1).*(x>=rho1 & x<1)+ ...
%          (1+rho1-x).*(x >=1 & x<1+rho1);   % nu_{2} or h_{1}
% nu = @(x) x.*pdf('unif',x,0,rho1) + ...
%          (1-rho1).*pdf('unif',x,rho1,1)+ (1+rho-x).*pdf('unif',x,1,1+rho1); 
% fplot(nu,[-0.5,2]) with support [0,1+rho1]
% for  rho1=0.5 the support = [0,3/2] 
%  ,,  rho1=0.7 the support = [0,1.7] 
%  ,,  rho1=0.9 the support = [0,1.9] 
%================================================================================
% WARNING: The following is not valid if rho1 in [0,-1/2+sqrt(5)/2=0.618,1]
% for the case rho1+rho1^2 <= 1

% a1 =@(x)  x.^2/(2*rho1^3);  
% a2= @(x) x./rho1 - rho1/2;
% a3= @(x) 1-rho1/2 + (1+rho1)*(x-rho1)./rho1^2 - (x.^2-rho1^2)./(2*rho1^3);
% a5= @(x) 1-rho1/2 + (rho1^2 - (x-1).^2./rho1^2)./(2*rho1);                
% a6= @(x) 1+  rho1/2 -(x-1)./rho1;
% a7 = @(x)(1+rho1)*(1+rho1 -(x-1)./rho1)/rho1 - ...
%          ((1+rho1)^2 - (x-1).^2./rho1^2)./(2*rho1);

% nu = @(x) (a1(x)*rho1^2).*pdf('unif',x,0,rho1^2) + ...
%           (a2(x)*(rho1-rho1^2)).*pdf('unif',x,rho1^2,rho1)+ ...
%           (a3(x)*rho1^2).*pdf('unif',x,rho1,rho1+rho1^2) + ...
%           (1-rho1-rho1^2).*pdf('unif',x,rho1+rho1^2,1) + ...
%           (a5(x)*rho1^2).*pdf('unif',x,1,1+rho1^2) + ...
%           (a6(x)*(rho1-rho1^2)).*pdf('unif',x,1+rho1^2,1+rho1) + ...
%           (a7(x)*rho1^2).*pdf('unif',x,1+rho1, 1+rho1+rho1^2); % nu_{3} or h_{2}
%
% fplot(nu,[-1,5]) with support [0,1+rho1+rho1^2]
% for  rho1=0.4 the support = [0,1.56] and support of the target [0,1.56/(1-0.4^3)]=[0,1.6667]
%  ,,  rho1=0.5 the support = [0,1.75]            ,,             [0,1.75/(1-0.5^3)]=[0,2]
%  ,,  rho1=0.6 the support = [0,1.96]            ..             [0,1.96/(1-0.6^3)]=[0,2.5] 

% nu = @(x) (a1(x)*rho^2).*unifpdf(x,0,rho^2) + ...
%           (a2(x)*(rho-rho^2)).*unifpdf(x,rho^2,rho)+ ...
%           (a3(x)*rho^2).*unifpdf(x,rho,rho+rho^2) + ...
%           (1-rho-rho^2).*unifpdf(x,rho+rho^2,1)  + ... 
%           (a5(x)*rho^2).*unifpdf(x,1,1+rho^2) + ...
%           (a6(x)*(rho-rho^2)).*unifpdf(x,1+rho^2, 1+rho) + ...
%           (a7(x)*rho^2).*unifpdf(x,1+rho, 1+rho+rho^2);  %  nu_{3} or h_{2}
% nu = @(x) (a1(x)).*(x>=0 & x<rho1^2) +(a2(x)).*(x>=rho1^2 & x<rho1)+ ...
%           (a3(x)).*(x>=rho1 & x<rho1+rho1^2) + ...
%           (x >=rho1+rho1^2 & x<1) + (a5(x)).*(x>=1 & x<1+rho1^2) + ...
%           (a6(x)).*(x>=1+rho1^2 & x<1+rho1) + ...
%           (a7(x)).*(x>= 1+rho1 & x<1+rho1+rho1^2); % nu_{3} or h_{2}
% fplot(nu,[-0.5,3]) 

%====================================================================
% WARNING: The following is not valid if rho1 [ -1/2+sqrt(5)/2=0.618,1]
% for the case  rho1+rho1^2 > 1
a1 = @(x) x.^2/(2*rho1^3); 
a2 = @(x) x./rho1 - rho1/2;
a3 = @(x) 1-rho1/2 + (1+rho1)*(x-rho1)./rho1^2 - (x.^2-rho1^2)./(2*rho1^3); 
a4 = @(x) x.*(1/rho1 + 1/rho1^2 + 1/rho1^3) - ...
          (x.^2)/rho1^3 -rho1/2 - 1/(2*rho1) -1/(2*rho1^3);
a5 = @(x) 1- (x-1).^2./(2*rho1^3);                                         
a6 = @(x) 1+  rho1/2 -(x-1)./rho1;
a7 = @(x)(1+rho1)*(1+rho1 -(x-1)./rho1)/rho1 - ...
         ((1+rho1)^2 - (x-1).^2./rho1^2)./(2*rho1);
         
% nu = @(x) (a1(x)*rho1^2).*pdf('unif',x,0,rho1^2) + ...
%           (a2(x)*(rho1-rho1^2)).*pdf('unif',x,rho1^2,rho1) + ...
%           (a3(x)*rho1^2).*pdf('unif',x,rho1,rho1+rho1^2) + ...
%           (1-rho1-rho1^2).*pdf('unif',x,rho1+rho1^2,1) + ...
%           (a5(x)*rho1^2).*pdf('unif',x,1,1+rho1^2) + ...
%           (a6(x)*(rho1-rho1^2)).*pdf('unif',x,1+rho1^2, 1+rho1) + ...
%           (a7(x)*rho1^2).*pdf('unif',x,1+rho1, 1+rho1+rho1^2);  % nu_{3} or h_{2}

% fplot(nu,[-1,5]) with support [0,1+rho1+rho1^2]
% for  rho1=0.7 the support = [0,2.19] and support of the target [0,2.19/(1-0.7^3)]=[0,3.333]
%  ,,  rho1=0.8 the support = [0,2.44]            ,,             [0,2.44/(1-0.8^3)]=[0,5]
%  ,,  rho1=0.9 the support = [0,2.71]            ,,             [0,2.71/(1-0.9^3)]=[0,10] 

nu = @(x) (a1(x)).*(x>=0 & x<rho1^2) + ...
          (a2(x)).*(x>=rho1^2 & x<rho1) + ...
          (a3(x)).*(x>=rho1 & x<1) + (a4(x)).*(x <rho1+rho1^2 & x>=1) + ...
          (a5(x)).*(x>=rho1+rho1^2 & x<1+rho1^2) + ...
          (a6(x)).*(x>=1+rho1^2 & x<1+rho1) + ...
          (a7(x)).*(x>= 1+rho1 & x<1+rho1+rho1^2); % nu_{3} or h_{2}
% fplot(nu,[-0.5,3])

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The order of the innovations:
m = 2;

% Define the function "V" (the value of "m" is globally defined)
V = @(x) 1+abs(x)^m;

% deltak = 1/V(k);
deltak   = 0.02;

% Values of "kmin" and "kmax"
kmin = 0;
kmax = 10; 

qmax = (kmax-kmin)/deltak;

% grid points
xk = zeros(qmax+1);
for i=1:qmax+1
    xk(i) = kmin+(i-1)*deltak;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Construction of the matrix "B_{k}"
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialization: 
Bk = zeros(qmax+1,qmax+1); % All rows and columns except the last ones

% All rows and columns except the last ones
for i=1:qmax 
    nuik = @(y) min(nu(y-rho*xk(i)), nu(y-rho*xk(i+1))); % Define nu_{i,k} 
    for j=1:qmax
        Bk(i,j) = quad(nuik,xk(j),xk(j+1));
    end
end

% Choose the column j_{0}\in {1,...,qkmax} and set column j_{0} of B_{k} 
j0 = floor(qmax/2);
for i=1:qmax 
    Bk(i,j0+1) = Bk(i,j0+1) + 1 - integral(nuik,kmin,kmax); 
end
Bk(qmax+1,j0+1) = 1;

% Compute the invariant probabilities "B_{k}"
pik = stationary(Bk);

% Define the pdf "p_{k}"
% set-up a grid
x  = kmin:(kmax-kmin)/100:kmax;
pk = [];
for y = kmin:(kmax-kmin)/100:kmax  
    % Compute pk(y)
    val = 0;
    for i=1:qmax 
        nuik = @(y) min(nu(y-rho*xk(i)), nu(y-rho*xk(i+1))); % Define nu_{i,k}
        val  = val + pik(i)*nuik(y);
    end
    pk = [pk,val];
end

% Figure 4 (Unif), 3rd figure from top. Case: rho = 0.9
% Plot the density 
dessin = plot(x,pk,'r',x,nu(x),'b');
hold on 
legend(dessin,'estimated invariant pdf','innovation pdf');
hold off;

%%%%%%%%%%%%%%%%%%%%  END MAIN %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
